'use strict';
var chai = require('chai');
var ZSchema = require('z-schema');
var customFormats = module.exports = function(zSchema) {
  // Placeholder file for all custom-formats in known to swagger.json
  // as found on
  // https://github.com/OAI/OpenAPI-Specification/blob/master/versions/2.0.md#dataTypeFormat

  var decimalPattern = /^\d{0,8}.?\d{0,4}[0]+$/;

  /** Validates floating point as decimal / money (i.e: 12345678.123400..) */
  zSchema.registerFormat('double', function(val) {
    return !decimalPattern.test(val.toString());
  });

  /** Validates value is a 32bit integer */
  zSchema.registerFormat('int32', function(val) {
    // the 32bit shift (>>) truncates any bits beyond max of 32
    return Number.isInteger(val) && ((val >> 0) === val);
  });

  zSchema.registerFormat('int64', function(val) {
    return Number.isInteger(val);
  });

  zSchema.registerFormat('float', function(val) {
    // better parsing for custom "float" format
    if (Number.parseFloat(val)) {
      return true;
    } else {
      return false;
    }
  });

  zSchema.registerFormat('date', function(val) {
    // should parse a a date
    return !isNaN(Date.parse(val));
  });

  zSchema.registerFormat('dateTime', function(val) {
    return !isNaN(Date.parse(val));
  });

  zSchema.registerFormat('password', function(val) {
    // should parse as a string
    return typeof val === 'string';
  });
};

customFormats(ZSchema);

var validator = new ZSchema({});
var request = require('request');
var expect = chai.expect;

describe('/users', function() {
  describe('get', function() {
  
  
  });

  describe('post', function() {
    it('should respond with 201 OK and should work with all required properties', function(done) {
      /*eslint-disable*/
      var schema = {
        "type": "object",
        "properties": {
          "id": {
            "type": "integer",
            "minimum": 0
          },
          "name": {
            "type": "string"
          },
          "age": {
            "type": "integer",
            "minimum": 0
          }
        },
        "required": [
          "id",
          "name",
          "age"
        ]
      };

      /*eslint-enable*/
      request({
        url: 'http://localhost:8080/v1/users',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":69335538,"name":"fugiat","age":36629857}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect(res.statusCode).to.equal(201);

		expect(validator.validate(body, schema)).to.be.true;
        done();
      });
    });

  
  
    it('should respond with default undefined and should not work without required property: id', function(done) {
      request({
        url: 'http://localhost:8080/v1/users',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"name":"laborum enim","age":54204302}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([201,400,409]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work without required property: name', function(done) {
      request({
        url: 'http://localhost:8080/v1/users',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":8002892,"age":25002623}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([201,400,409]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work without required property: age', function(done) {
      request({
        url: 'http://localhost:8080/v1/users',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":43202852,"name":"mollit consectetu"}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([201,400,409]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'id\' of type \'boolean\'', function(done) {
      request({
        url: 'http://localhost:8080/v1/users',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":true,"name":"Lorem","age":37518073}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([201,400,409]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'id\' of type \'number\'', function(done) {
      request({
        url: 'http://localhost:8080/v1/users',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":60.04,"name":"eu est in non","age":22617824}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([201,400,409]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'id\' of type \'null\'', function(done) {
      request({
        url: 'http://localhost:8080/v1/users',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":null,"name":"Duis","age":89727642}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([201,400,409]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'id\' of type \'string\'', function(done) {
      request({
        url: 'http://localhost:8080/v1/users',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":"vz&xxRXIzQ","name":"anim dolor nostrud","age":55025193}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([201,400,409]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not pass validation for minimum of property: id', function(done) {
      request({
        url: 'http://localhost:8080/v1/users',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":-1,"name":"sed adipisicing nostrud","age":26597424}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([201,400,409]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'name\' of type \'boolean\'', function(done) {
      request({
        url: 'http://localhost:8080/v1/users',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":93268219,"name":false,"age":79699625}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([201,400,409]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'name\' of type \'integer\'', function(done) {
      request({
        url: 'http://localhost:8080/v1/users',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":89252060,"name":6528053484191744,"age":71829361}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([201,400,409]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'name\' of type \'number\'', function(done) {
      request({
        url: 'http://localhost:8080/v1/users',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":53395629,"name":78.02,"age":87869490}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([201,400,409]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'name\' of type \'null\'', function(done) {
      request({
        url: 'http://localhost:8080/v1/users',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":7838233,"name":null,"age":34518057}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([201,400,409]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'age\' of type \'boolean\'', function(done) {
      request({
        url: 'http://localhost:8080/v1/users',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":78346466,"name":"sed","age":true}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([201,400,409]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'age\' of type \'number\'', function(done) {
      request({
        url: 'http://localhost:8080/v1/users',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":62370153,"name":"incididunt magna tempor dolore","age":46.28}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([201,400,409]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'age\' of type \'null\'', function(done) {
      request({
        url: 'http://localhost:8080/v1/users',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":91388032,"name":"eiusmod","age":null}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([201,400,409]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'age\' of type \'string\'', function(done) {
      request({
        url: 'http://localhost:8080/v1/users',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":40207280,"name":"culpa consequat veni","age":"ap0IfibvDRG1K)W4"}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([201,400,409]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not pass validation for minimum of property: age', function(done) {
      request({
        url: 'http://localhost:8080/v1/users',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":11319994,"name":"tempor qui cupidatat","age":-1}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([201,400,409]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });

  });

});
