'use strict';
var chai = require('chai');
var ZSchema = require('z-schema');
var customFormats = module.exports = function(zSchema) {
  // Placeholder file for all custom-formats in known to swagger.json
  // as found on
  // https://github.com/OAI/OpenAPI-Specification/blob/master/versions/2.0.md#dataTypeFormat

  var decimalPattern = /^\d{0,8}.?\d{0,4}[0]+$/;

  /** Validates floating point as decimal / money (i.e: 12345678.123400..) */
  zSchema.registerFormat('double', function(val) {
    return !decimalPattern.test(val.toString());
  });

  /** Validates value is a 32bit integer */
  zSchema.registerFormat('int32', function(val) {
    // the 32bit shift (>>) truncates any bits beyond max of 32
    return Number.isInteger(val) && ((val >> 0) === val);
  });

  zSchema.registerFormat('int64', function(val) {
    return Number.isInteger(val);
  });

  zSchema.registerFormat('float', function(val) {
    // better parsing for custom "float" format
    if (Number.parseFloat(val)) {
      return true;
    } else {
      return false;
    }
  });

  zSchema.registerFormat('date', function(val) {
    // should parse a a date
    return !isNaN(Date.parse(val));
  });

  zSchema.registerFormat('dateTime', function(val) {
    return !isNaN(Date.parse(val));
  });

  zSchema.registerFormat('password', function(val) {
    // should parse as a string
    return typeof val === 'string';
  });
};

customFormats(ZSchema);

var validator = new ZSchema({});
var request = require('request');
var expect = chai.expect;

describe('/users/{userId}', function() {
  describe('get', function() {
    it('should respond with 200 OK', function(done) {
      /*eslint-disable*/
      var schema = {
        "type": "object",
        "properties": {
          "id": {
            "type": "integer",
            "minimum": 0
          },
          "name": {
            "type": "string"
          },
          "age": {
            "type": "integer",
            "minimum": 0
          }
        },
        "required": [
          "id",
          "name",
          "age"
        ]
      };

      /*eslint-enable*/
      request({
        url: 'http://localhost:8080/v1/users/1',
        json: true,
        method: 'GET',
        headers: {
          'Content-Type': 'application/json'
        }
      },
      function(error, res, body) {
        if (error) return done(error);

        expect(res.statusCode).to.equal(200);

		expect(validator.validate(body, schema)).to.be.true;
        done();
      });
    });

    it('should respond with 404 Not found', function(done) {
      request({
        url: 'http://localhost:8080/v1/users/8',
        json: true,
        method: 'GET',
        headers: {
          'Content-Type': 'application/json'
        }
      },
      function(error, res, body) {
        if (error) return done(error);

        expect(res.statusCode).to.equal(404);

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });

  
  });

  describe('put', function() {
    it('should respond with 200 OK and should work with all required properties', function(done) {
      /*eslint-disable*/
      var schema = {
        "type": "object",
        "properties": {
          "id": {
            "type": "integer",
            "minimum": 0
          },
          "name": {
            "type": "string"
          },
          "age": {
            "type": "integer",
            "minimum": 0
          }
        },
        "required": [
          "id",
          "name",
          "age"
        ]
      };

      /*eslint-enable*/
      request({
        url: 'http://localhost:8080/v1/users/5',
        json: true,
        method: 'PUT',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":44373424,"name":"nulla sint sunt","age":83551063}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect(res.statusCode).to.equal(200);

		expect(validator.validate(body, schema)).to.be.true;
        done();
      });
    });

  
    it('should respond with 404 Not found', function(done) {
      request({
        url: 'http://localhost:8080/v1/users/90',
        json: true,
        method: 'PUT',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":44373424,"name":"nulla sint sunt","age":83551063}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect(res.statusCode).to.equal(404);

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });

    it('should respond with default undefined and should not work without required property: id of body', function(done) {
      request({
        url: 'http://localhost:8080/v1/users/5',
        json: true,
        method: 'PUT',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"name":"esse officia consectetur","age":79933143}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400,404]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work without required property: name of body', function(done) {
      request({
        url: 'http://localhost:8080/v1/users/5',
        json: true,
        method: 'PUT',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":7083405,"age":42049499}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400,404]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work without required property: age of body', function(done) {
      request({
        url: 'http://localhost:8080/v1/users/5',
        json: true,
        method: 'PUT',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":14853604,"name":"anim cupidatat elit"}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400,404]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'id\' of type \'boolean\' of body', function(done) {
      request({
        url: 'http://localhost:8080/v1/users/5',
        json: true,
        method: 'PUT',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":true,"name":"qui tempor","age":13915226}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400,404]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'id\' of type \'number\' of body', function(done) {
      request({
        url: 'http://localhost:8080/v1/users/5',
        json: true,
        method: 'PUT',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":88.84,"name":"voluptate velit dolor","age":30894639}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400,404]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'id\' of type \'null\' of body', function(done) {
      request({
        url: 'http://localhost:8080/v1/users/5',
        json: true,
        method: 'PUT',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":null,"name":"cupidatat sunt nisi et","age":26203820}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400,404]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'id\' of type \'string\' of body', function(done) {
      request({
        url: 'http://localhost:8080/v1/users/5',
        json: true,
        method: 'PUT',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":"Ce0D9z","name":"occaecat ipsum","age":23511035}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400,404]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not pass validation for minimum of property: id of body', function(done) {
      request({
        url: 'http://localhost:8080/v1/users/5',
        json: true,
        method: 'PUT',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":-1,"name":"in qui officia cupidatat","age":21614572}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400,404]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'name\' of type \'boolean\' of body', function(done) {
      request({
        url: 'http://localhost:8080/v1/users/5',
        json: true,
        method: 'PUT',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":19939791,"name":true,"age":43820351}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400,404]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'name\' of type \'integer\' of body', function(done) {
      request({
        url: 'http://localhost:8080/v1/users/5',
        json: true,
        method: 'PUT',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":10110824,"name":-1355694133477376,"age":93973612}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400,404]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'name\' of type \'number\' of body', function(done) {
      request({
        url: 'http://localhost:8080/v1/users/5',
        json: true,
        method: 'PUT',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":39935689,"name":10.8,"age":38029330}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400,404]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'name\' of type \'null\' of body', function(done) {
      request({
        url: 'http://localhost:8080/v1/users/5',
        json: true,
        method: 'PUT',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":24072146,"name":null,"age":47879837}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400,404]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'age\' of type \'boolean\' of body', function(done) {
      request({
        url: 'http://localhost:8080/v1/users/5',
        json: true,
        method: 'PUT',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":67718639,"name":"commodo magna et dolor","age":true}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400,404]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'age\' of type \'number\' of body', function(done) {
      request({
        url: 'http://localhost:8080/v1/users/5',
        json: true,
        method: 'PUT',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":68743746,"name":"aute elit ullamco","age":99.19}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400,404]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'age\' of type \'null\' of body', function(done) {
      request({
        url: 'http://localhost:8080/v1/users/5',
        json: true,
        method: 'PUT',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":99089891,"name":"cupidatat anim","age":null}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400,404]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'age\' of type \'string\' of body', function(done) {
      request({
        url: 'http://localhost:8080/v1/users/5',
        json: true,
        method: 'PUT',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":40147592,"name":"sunt voluptate sed Excepteur","age":"R9fxz#pQ7v"}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400,404]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not pass validation for minimum of property: age of body', function(done) {
      request({
        url: 'http://localhost:8080/v1/users/5',
        json: true,
        method: 'PUT',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"id":33491234,"name":"laborum","age":-1}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400,404]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });

  });

  describe('delete', function() {
    it('should respond with 200 OK', function(done) {
      /*eslint-disable*/
      var schema = {
        "type": "object",
        "properties": {
          "id": {
            "type": "integer",
            "minimum": 0
          },
          "name": {
            "type": "string"
          },
          "age": {
            "type": "integer",
            "minimum": 0
          }
        },
        "required": [
          "id",
          "name",
          "age"
        ]
      };

      /*eslint-enable*/
      request({
        url: 'http://localhost:8080/v1/users/10',
        json: true,
        method: 'DELETE',
        headers: {
          'Content-Type': 'application/json'
        }
      },
      function(error, res, body) {
        if (error) return done(error);

        expect(res.statusCode).to.equal(200);

		expect(validator.validate(body, schema)).to.be.true;
        done();
      });
    });

    it('should respond with 404 Not found', function(done) {
      request({
        url: 'http://localhost:8080/v1/users/11',
        json: true,
        method: 'DELETE',
        headers: {
          'Content-Type': 'application/json'
        }
      },
      function(error, res, body) {
        if (error) return done(error);

        expect(res.statusCode).to.equal(404);

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });

  
  });

});
